// ******************************************************************************
// * Licensed Materials - Property of IBM
// * 5724-W86, 5724-X63
// *  Copyright IBM Corporation 2008, 2012. All Rights Reserved.
// * US Government Users Restricted Rights- Use, duplication or disclosure
// * restricted by GSA ADP Schedule Contract with IBM Corp.
// ******************************************************************************
// *
// ******************************************************************************

var cardlevelreports = "FALSE";
if (PV.configuration.CARDLEVEL == "TRUE")
        cardlevelreports = "TRUE";

/**
 * Returns the value, and if it's not a number it returns 0
 * @param {Object} value
 * @return {Number} valueAsNumber
 */
function ZeroIfEmpty(value) {
        logMessage(4, "ZeroIfEmpty", value);
	if (value == undefined)
		return 0;

        if (value == nil)
                return 0;

        if (value.toString() == "NaN")
                return 0;

        return value;
}

/**
 * The class structure to use with APModeler.js it contains an array of objects,
 * keyed by name, each of these objects has a filter function, a timestamp function,
 * a properties array (of functions) and a metrics array (of functions)
 * @param {Object} record
 */

function genDeviceName(descr) {
	var name = descr.substring(0, descr.lastIndexOf("_"));    
	return name;   
}


var ethSpeedCache = new Object();


		
var subElementClasses = {
	"Alcatel5529SDC_Port": 
	{
 	   // The filter function returns true if a record conforms to the Alcatel5529SDC_Port class
	   filter: function(record) {
			return true;
	   },
	   // The timestamp function returns the timestamp to use when creating metrics
	   timestamp: function(record) {
		return (record.Timestamp);
	   },
	   // Class properties
	   properties:	{
                "Alcatel5529SDC_CardID": function(record) {
                     if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
                         return record.Value;
                     }
                },
				// Flag that the subelement was produce by the Alcatel5529SDC_UBAAdaptor provider
				"Alcatel5529SDC_UBAAdaptor": function(record) {
					return "true";
				},
				// Flag that the subelement conforms to the Alcatel5529SDC_Port class
				"Alcatel5529SDC_Port": function(record) {
					return "true";
				},
				"Alcatel5529SDC_Port_Type": function(record) {
					if (record.CounterID == "58")
						return record.Value;
				},
				"Alcatel5529SDC_Port_Slot": function(record) {
					if ((record.CounterID == "58") || record.CounterID == "13131")
						return record.ObjectID.split("-")[2];
				},
				"Alcatel5529SDC_Port_Shelf": function(record) {
					if ((record.CounterID == "58")|| record.CounterID == "13131")
						return record.ObjectID.split("-")[1];
				},
				"Alcatel5529SDC_Port_Rack": function(record) {
					if ((record.CounterID == "58")|| record.CounterID == "13131")
						return record.ObjectID.split("-")[0];
				},
				"Alcatel5529SDC_Port_Port": function(record) {
					if ((record.CounterID == "58" || record.CounterID == "13131") && cardlevelreports == "FALSE")
						return record.ObjectID.split("-")[3];
				},
				//TNPM1.3D: Extract Device Name from CounterID 371
			//    "Alcatel5529SDC_DeviceName": function(record) {
				    //TNPM1.3D: Commented out
					//return genDeviceName(record.descriptorName);
			//		if( record.CounterID == "371" && isValid(record.Value) ) {
			//			return record.Value;
			//		}
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
			    //TNPM1.3D: Extract Device IP from CounterID 370
			    "Alcatel5529SDC_DeviceIP": function(record) {			    	
			    	if( record.CounterID == "370" && isValid(record.Value) ) {
						return record.Value;
					}
			    },
				"Alcatel5529SDC_CARD": function(record)  {
                     if (record.CounterID == "111191")  
                        return "true";
                },
				"Alcatel5529SDC_Port_Card": function(record) {
					if ((record.CounterID == "58")|| record.CounterID == "13131" || record.CounterID == "111191")  
						return record.ObjectID.substring(0, record.ObjectID.lastIndexOf("-"));
		                }
		
			   },
		   // Class formulas
        formulas: {
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~CPU~CPU Utilization": function(record) {
	                  logMessage(4, "CPU", "record: " + record);
	                  if (record.CounterID == "111191") {
	                          logMessage(4, "CPU", "record.CounterID: " + record.CounterID);
	                          var formulaPath = arguments.callee.formulaPath;
	                          if (record.Value == undefined)   {                                   
	                          return; }
	                          else { 
	                          var inputValue = record.Value.toString();
	                          //var hexStr = inputValue.substring(0,3); 
	                          var hexStr = inputValue.substr(0,2); 
	                          var newValue = parseInt(hexStr,16); 
	                          return newValue;    }                          
	                          //return delta(record,formulaPath);
	                  }
	          },
             "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~Memory~Total Memory": function(record) {
                      if (record.CounterID == "111194") {
                              var formulaPath = arguments.callee.formulaPath;
                          //    return delta(record,formulaPath);
                              return record.Value;
                      }
              },
             "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~Memory~Memory Used": function(record) {
                      if (record.CounterID == "111195") {
                              var formulaPath = arguments.callee.formulaPath;
                          //    return delta(record,formulaPath);
                              return record.Value;
                      }
              }
		   }
	   },
	"Alcatel5529SDC_Ethernet": 
	{
		// The filter function returns true if a record conforms to the Alcatel5529SDC_Ethernet class
		filter: function(record) {
                                return true;
		},
		// The timestamp function returns the timestamp to use when creating metrics
		timestamp: function(record) {
                        return (record.Timestamp);
		},
		// Class properties
		properties:	{
				//TNPM1.3D: Extract Device Name from CounterID 371
			//    "Alcatel5529SDC_DeviceName": function(record) {
				    //TNPM1.3D: Commented out
					//return genDeviceName(record.descriptorName);
			//		if( record.CounterID == "371" && isValid(record.Value) ) {
			//			return record.Value;
			//		}
			//   },	
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
			    //TNPM1.3D: Extract Device IP from CounterID 370
			    "Alcatel5529SDC_DeviceIP": function(record) {			    	
			    	if( record.CounterID == "370" && isValid(record.Value) ) {
						return record.Value;
					}
			    },
				"Alcatel5529SDC_CardID": function(record) {
					if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
						return record.Value;
					}
				},
				// Flag that the subelement was produce by the Alcatel5529SDC_UBAAdaptor provider
				"Alcatel5529SDC_UBAAdaptor": function(record) {
					logMessage(4, "ETHERNET_ADAPTOR", "adaptor...");
					return "true";
				},
				// Flag that the subelement conforms to the Alcatel5529SDC_Ethernet class
				"Alcatel5529SDC_Ethernet": function(record) {
					logMessage(4, "ETHERNET_ETHERNET", "adaptor...");
					if ((record.CounterID == "58" && record.Value == "ETHERNET") || record.CounterID == "13131")
						return "true";
				},
				"Alcatel5529SDC_Ethernet_Speed": function(state) {
					// logMessage(4, "ETHERNET_SPEED", "Setting ethernet port speed....");
					// logMessage(4, "ETHERNET_SPEED", "num: " + sumCurrentPeriod(state, 90011));
					// logMessage(4, "ETHERNET_SPEED", "state: " + state);
					// return sumCurrentPeriod(state,90011);
					logMessage(4, "ETHERNET_SE", "eth: " + state.subelement);
					return ethSpeedCache[state.subelement];
				}
		},
		
		// Class formulas
		formulas: {
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Unknown protocol errors": function(record) {	
				if (record.CounterID == "90010")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Outgoing unicast packets": function(record) {
				if (record.CounterID == "90008")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Outgoing packet errors": function(record) {
				if (record.CounterID == "90006")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Outgoing octets": function(record) {
				if (record.CounterID == "90002")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Outgoing non-unicast packets": function(record) {
				if (record.CounterID == "90013")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Outgoing discarded packets": function(record) {
				if (record.CounterID == "90004")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Incoming unicast packets": function(record) {
				if (record.CounterID == "90007")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Incoming packet errors": function(record) {
				if (record.CounterID == "90005")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Incoming octets": function(record) {
				if (record.CounterID == "90001")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Incoming non-unicast packets": function(record) {
				if (record.CounterID == "90012")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Incoming discarded packets": function(record) {
				if (record.CounterID == "90003")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Incoming multicast packets": function(record) {
				if (record.CounterID == "80001")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Outgoing multicast packets": function(record) {
				if (record.CounterID == "80002")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Incoming broadcast packets": function(record) {
				if (record.CounterID == "80003")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet~Outgoing broadcast packets": function(record) {
				if (record.CounterID == "80004")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			}
		}
	},
	"Alcatel5529SDC_XDSL": 
	{
		// The filter function returns true if a record conforms to the Alcatel5529SDC_XDSL class
		filter: function(record) {
                        // var regExp = new RegExp("^[0-9]+\-[0-9]+\-[0-9]+\-[0-9]+$");
                        if (seRegExp.exec(record.ObjectID) != null && cardlevelreports == "FALSE")
                                return true;
		 },
		
		// The timestamp function returns the timestamp to use when creating metrics
		timestamp: function(record) {
                        return (record.Timestamp);
		 },
		
		// Class properties
		properties: {
		 	//TNPM1.3D: Extract Device Name from CounterID 371
		//    "Alcatel5529SDC_DeviceName": function(record) {
			    //TNPM1.3D: Commented out
				//return genDeviceName(record.descriptorName);
		//		if( record.CounterID == "371" && isValid(record.Value) ) {
		//			return record.Value;
		//		}
		//   },	
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
		    //TNPM1.3D: Extract Device IP from CounterID 370
		    "Alcatel5529SDC_DeviceIP": function(record) {			    	
		    	if( record.CounterID == "370" && isValid(record.Value) ) {
					return record.Value;
				}
		    },
            "Alcatel5529SDC_CardID": function(record) {
                 if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
                     return record.Value;
                 }
            },
			// Flag that the subelement was produce by the Alcatel5529SDC_UBAAdaptor provider
			"Alcatel5529SDC_UBAAdaptor":  function(record) {
				return "true";
			},
			
			// Flag that the subelement conforms to the Alcatel5529SDC_XDSL class
			"Alcatel5529SDC_XDSL": function(record) {
				if (record.CounterID == "58" && record.Value == "XDSL")
					return "true";
			}
		},
		
		// Class formulas
		formulas: {
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Unavailable Seconds Upstream": function(record) {
				if (record.CounterID == "1231")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Unavailable Seconds Downstream": function(record) {
				if (record.CounterID == "1232")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Severely Errored Seconds (SES) Upstream": function(record) {
				if (record.CounterID == "90")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Severely Errored Seconds (SES) Downstream": function(record) {
				if (record.CounterID == "89")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Loss of Signal Failures Upstream": function(record) {
				if (record.CounterID == "1237")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Loss of Signal Failures Downstream": function(record) {
				if (record.CounterID == "1236")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Loss of Power Failures Upstream": function(record) {
				if (record.CounterID == "1261")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Loss of Power Failures Downstream": function(record) {
				if (record.CounterID == "1247")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Loss of Link seconds": function(record) {
				if (record.CounterID == "1249")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Loss of Framing Failures Upstream": function(record) {
				if (record.CounterID == "1245")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Loss of Framing Failures Downstream": function(record) {
				if (record.CounterID == "1244")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Errored Seconds Upstream": function(record) {
				if (record.CounterID == "88")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~Errored Seconds Downstream": function(record) {
				if (record.CounterID == "87")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~ATM Unknown protocol errors": function(record) {
				if (record.CounterID == "90110")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~ATM Outgoing packet errors": function(record) {
				if (record.CounterID == "90106")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~ATM Outgoing octets": function(record) {
				if (record.CounterID == "90102")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~ATM Incoming packet errors": function(record) {
				if (record.CounterID == "90105")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~ATM Incoming octets": function(record) {
				if (record.CounterID == "90101")
				{
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~NE US Actual Bit Rate": function(record)	{
				if (record.CounterID == "15")
					return record.Value;
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~XDSL~FE DS Actual Bit Rate": function(record)	{
				if (record.CounterID == "16")
					return record.Value;
			}
		}
	},
	// Alcatel5529SDC_VLAN is used for total/overall VLAN stats
	"Alcatel5529SDC_VLAN": 
	{
                // The filter function returns true if a record conforms to the Alcatel5529SDC_VLAN class
                filter: function(record) 
				{

/*

		                // var regExp = new RegExp("^(1485|1455|1459|1463|1467|1471|1475|1479|1483)$");
                        	if (seRegExp.exec(record.CounterID) != null)
					{
                                        // we don't want to return true if it's Bridge Port VLAN association
                                        if (record.CounterID != "1485")
                                                return true;
                                        else
                                                {
                                                // if the ObjectID contains a space, it is probably Bridge Port VLAN Assoc
                                                // in this case, return nothing, otherwise return true
                                                if (record.ObjectID.split(" ").length==1)
                                                        return true;
                                                }
					}

*/

				return true;
                		},

                // The timestamp function returns the timestamp to use when creating metrics
                timestamp: function(record) {
                        return (record.Timestamp);
                },

                // Class properties
                properties:     {
                	//TNPM1.3D: Extract Device Name from CounterID 371
        		//    "Alcatel5529SDC_DeviceName": function(record) {
        			    //TNPM1.3D: Commented out
        				//return genDeviceName(record.descriptorName);
        		//		if( record.CounterID == "371" && isValid(record.Value) ) {
        		//			return record.Value;
        		//		}
        		//    },	
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
        		    //TNPM1.3D: Extract Device IP from CounterID 370
        		    "Alcatel5529SDC_DeviceIP": function(record) {			    	
        		    	if( record.CounterID == "370" && isValid(record.Value) ) {
        					return record.Value;
        				}
        		    },
                        "Alcatel5529SDC_CardID": function(record) {
                             if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
                                 return record.Value;
                             }
                        },
                        // Flag that the subelement was produce by the Alcatel5529SDC_UBAAdaptor provider
                        "Alcatel5529SDC_UBAAdaptor": function(record) {
                                return "true";
                        },

                        // Flag that the subelement conforms to the Alcatel5529SDC_VLAN class
                        "Alcatel5529SDC_VLAN": function(record) {
                        	if (record.CounterID=="1485" && record.ObjectID.split(" ").length==1)
					return true;
                        },

                        // VLAN circuit
                        "Alcatel5529SDC_VLAN_circuit": function(record) {
                                if (record.CounterID == "1485" && record.ObjectID.split(" ").length==1)
                                        return record.Value;
                        },

                        // bridgeport index
                        "Alcatel5529SDC_BridgePortIndex": function(record) {
                                if (record.CounterID == "1485" && record.ObjectID.split(" ").length==1)
                                        {
                                        // this should be a single number
                                        return record.ObjectID;
                                        }
                        }
	
                },

                // Class formulas
                formulas: {
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Frame Traffic Summary~US Forwarded Frames": function(record)      {
                                
								if (record.CounterID == "1455")
								{
									var formulaPath = arguments.callee.formulaPath;
									return delta(record,formulaPath,true);
								}
                        },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Frame Traffic Summary~DS Forwarded Frames": function(record)      {
                                if (record.CounterID == "1459")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Frame Traffic Summary~US Discarded Frames": function(record)      {
                                if (record.CounterID == "1471")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Frame Traffic Summary~DS Discarded Frames": function(record)      {
                                if (record.CounterID == "1475")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Byte Traffic Summary~US Forwarded Bytes": function(record)      {
                                if (record.CounterID == "1463")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Byte Traffic Summary~DS Forwarded Bytes": function(record)      {
                                if (record.CounterID == "1467")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Byte Traffic Summary~US Discarded Bytes": function(record)      {
                                if (record.CounterID == "1479")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~VLAN~Byte Traffic Summary~DS Discarded Bytes": function(record)      {
                                if (record.CounterID == "1483")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        }
                }
        },
    // Alcatel5529SDC_BridgePort_VLAN is used for Bridge Port VLAN association stats
    "Alcatel5529SDC_BridgePort_VLAN": 
	{
                // The filter function returns true if a record conforms to the Alcatel5529SDC_BridgePort_VLAN class
                filter: function(record) 
		{

/*

                var regExp = new RegExp("^(1485|1402|1404|1406|1411|1415|1419|1423|1427|1431|1435|1439|1443)$");
                if (regExp.exec(record.CounterID) != null)
				{
                                // we don't want to return true if it's NOT Bridge Port VLAN association
                                if (record.CounterID != "1485")
                                        return true;
                                else
                                        {
                                        // if the ObjectID contains a space, it is probably Bridge Port VLAN Assoc
                                        // in this case, return true
                                        if (record.ObjectID.split(" ").length==2)
						{
                                                return true;
						}
                                        }
				}

*/
		return true;
                },

                // The timestamp function returns the timestamp to use when creating metrics
                timestamp: function(record) {
                        return (record.Timestamp);
                },

                // Class properties
                properties:     {
                	//TNPM1.3D: Extract Device Name from CounterID 371
        		//    "Alcatel5529SDC_DeviceName": function(record) {
        			    //TNPM1.3D: Commented out
        				//return genDeviceName(record.descriptorName);
        		//		if( record.CounterID == "371" && isValid(record.Value) ) {
        		//			return record.Value;
        		//		}
        		//    },	
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
        		    //TNPM1.3D: Extract Device IP from CounterID 370
        		    "Alcatel5529SDC_DeviceIP": function(record) {			    	
        		    	if( record.CounterID == "370" && isValid(record.Value) ) {
        					return record.Value;
        				}
        		    },
                        "Alcatel5529SDC_CardID": function(record) {
                             if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
                                 return record.Value;
                             }
                        },
                        // Flag that the subelement was produce by the Alcatel5529SDC_UBAAdaptor provider
                        "Alcatel5529SDC_UBAAdaptor": function(record) {
                                return "true";
                        },

                        // Flag that the subelement conforms to the Alcatel5529SDC_BridgePort_VLAN class
                        "Alcatel5529SDC_BridgePort_VLAN": function(record) {
                               // only return true if the ObjectID contains 2 items separated by a space
                               if (record.CounterID == "1485" && record.ObjectID.split(" ").length == 2)
                                       	return true;
                        },
			// VLAN circuit
			"Alcatel5529SDC_VLAN_circuit": function(record) {
				if (record.CounterID == "1485" && record.ObjectID.split(" ").length == 2)
					return record.Value;
			},
			// VLAN name
			"Alcatel5529SDC_VLAN_name": function(record) {
                                if (record.CounterID == "1402")
                                        return record.Value;
			},
			// bridgeport index
			"Alcatel5529SDC_BridgePortIndex": function(record) {
                                if (record.CounterID == "1402")
                                        {
                                        // two numbers separated by space, we only want the first one
                                        return record.ObjectID.split(" ")[0];
                                        }
                        },
                        // vlan index
                        "Alcatel5529SDC_VLANIndex": function(record) {
                                if (record.CounterID == "1402")
                                        {
                                        // two numbers separated by space, we only want the second one
                                        return record.ObjectID.split(" ")[1];
                                        }
                        }
                },

                // Class formulas
                formulas: 
			{
/*
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Health Summary~BP-VLAN Time Elapsed": function(record)      
				{
                                if (record.CounterID == "1406")
                                        return ZeroIfEmpty(record.Value);
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Health Summary~BP-VLAN Time Measured": function(record)      
				{
                                if (record.CounterID == "1411")
                                        return ZeroIfEmpty(record.Value);
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Health Summary~BP-VLAN number of valid intervals": function(record) 
				{
                                if (record.CounterID == "1404")
                                        return ZeroIfEmpty(record.Value);
                        	},
*/
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Frame Traffic Summary~BP-VLAN US Forwarded Frames": function(record)
				{
                                if (record.CounterID == "1415")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Frame Traffic Summary~BP-VLAN DS Forwarded Frames": function(record)
				{
                                if (record.CounterID == "1419")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Frame Traffic Summary~BP-VLAN US Discarded Frames": function(record)
				{
                                if (record.CounterID == "1431")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Frame Traffic Summary~BP-VLAN DS Discarded Frames": function(record) 
				{
                                if (record.CounterID == "1435")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Byte Traffic Summary~BP-VLAN US Forwarded Bytes": function(record)
				{
                                if (record.CounterID == "1423")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}///        return ZeroIfEmpty(delta(record));
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Byte Traffic Summary~BP-VLAN DS Forwarded Bytes": function(record)
				{
                                if (record.CounterID == "1427")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}//        return ZeroIfEmpty(delta(record));
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Byte Traffic Summary~BP-VLAN US Discarded Bytes": function(record)
				{
                                if (record.CounterID == "1439")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}///        return ZeroIfEmpty(delta(record));
                        	},
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Bridge Port VLAN Association~Byte Traffic Summary~BP-VLAN DS Discarded Bytes": function(record)
				{
                                if (record.CounterID == "1443")
                                {
										var formulaPath = arguments.callee.formulaPath;
										return delta(record,formulaPath,true);
								}///        return ZeroIfEmpty(delta(record));
                        	}
                	}
			// end of class formulas
 	},
	// end of Alcatel5529SDC_BridgePort_VLAN class       
 	// card level XDSL PDD R9
 	"Alcatel5529SDC_XDSL_CARD": 
 	{
        filter: function(record) 
		{
	        if (cardlevelreports == "TRUE")
	        	return true;
		},

		// The timestamp function returns the timestamp to use when creating metrics
		timestamp: function(record) 
		{
			return (record.Timestamp);
        },

        // Class properties
        properties: 
        {
        	//TNPM1.3D: Extract Device Name from CounterID 371
		//    "Alcatel5529SDC_DeviceName": function(record) {
			    //TNPM1.3D: Commented out
				//return genDeviceName(record.descriptorName);
		//		if( record.CounterID == "371" && isValid(record.Value) ) {
		//			return record.Value;
		//		}
		//   },	
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
		    //TNPM1.3D: Extract Device IP from CounterID 370
		    "Alcatel5529SDC_DeviceIP": function(record) {			    	
		    	if( record.CounterID == "370" && isValid(record.Value) ) {
					return record.Value;
				}
		    },
            "Alcatel5529SDC_CardID": function(record) {
                 if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
                     return record.Value;
                 }
            },
            // Flag that the subelement was produce by the Alcatel5529SDC_UBAAdaptor provider
	        "Alcatel5529SDC_UBAAdaptor":  function(record) 
			{
            	return "true";
			},

			// Flag that the subelement conforms to the Alcatel5529SDC_XDSL class
			"Alcatel5529SDC_XDSL_CARD": function(record) 
			{
				if (record.CounterID == "58" && record.Value == "XDSL" && cardlevelreports == "TRUE")
					return "true";
			},
			// Flag if the subelement is for cardlevel reports
			"Alcatel5529SDC_CARD": function(record)
            {
				if (cardlevelreports == "TRUE")
                    return "true";
            }
         },

         // Class formulas
         formulas: 
         {
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Unavailable Seconds Upstream": 
		function(record) 
			{
                        if (record.CounterID == "1231")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Unavailable Seconds Downstream": 
		function(record) 
			{
                        if (record.CounterID == "1232")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Severely Errored Seconds (SES) Upstream": 
		function(record) 
			{
                        if (record.CounterID == "90")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Severely Errored Seconds (SES) Downstream": 
		function(record) 
			{
                        if (record.CounterID == "89")
                       	{
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Loss of Signal Failures Upstream": 
		function(record) 
			{
                        if (record.CounterID == "1237")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
           },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Loss of Signal Failures Downstream": 
		function(record) 
			{
                        if (record.CounterID == "1236")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Loss of Power Failures Upstream": 
		function(record) 
			{
                        if (record.CounterID == "1261")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Loss of Power Failures Downstream": 
		function(record) 
			{
                        if (record.CounterID == "1247")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Loss of Link seconds": 
		function(record) 
			{
                        if (record.CounterID == "1249")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Loss of Framing Failures Upstream": 
		function(record) 
			{
                        if (record.CounterID == "1245")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Loss of Framing Failures Downstream": 
		function(record) 
			{
                        if (record.CounterID == "1244")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                 "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Errored Seconds Upstream": 
		function(record) 
			{
                        if (record.CounterID == "88")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~Errored Seconds Downstream": 
		function(record) 
			{
                        if (record.CounterID == "87")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~ATM Unknown protocol errors": 
		function(record) 
			{
                        if (record.CounterID == "90110")
                        {
							var formulaPath = arguments.callee.formulaPath;
							return delta(record,formulaPath);
						}
            },
                        "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~ATM Outgoing packet errors": 
		function(record) 
			{
            	if (record.CounterID == "90106")
                {
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~ATM Outgoing octets": 
		function(record) 
			{
            	if (record.CounterID == "90102")
                {
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~ATM Incoming packet errors": 
		function(record) 
			{
            	if (record.CounterID == "90105")
                {
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~ATM Incoming octets": 
		function(record) 
			{
            	if (record.CounterID == "90101")
                {
					var formulaPath = arguments.callee.formulaPath;
					return delta(record,formulaPath);
				}
            },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~NE US Actual Bit Rate": 
		function(record)      
			{
                        if (record.CounterID == "15")
                        	return record.Value;
                        },
                "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~XDSL~FE DS Actual Bit Rate": 
		function(record)      
			{
                        if (record.CounterID == "16")
                        	return record.Value;
                        }
                }
	}, // end XDSL card level class
	"Alcatel5529SDC_EthernetLAG": 
	{
		// The filter function returns true if a record conforms to the Alcatel5529SDC_LAGEthernet class
		filter: function(record) {
		// Since no records can be used from this particular function it will always return false,
		// however if the record is related to LAG then we'll add the isLagRelated = true;
			addIsLagRelatedToRecord(record);
			return false;
		},
		
		// The timestamp function returns the timestamp to use when creating metrics
		timestamp: function(record) {
                        return (record.Timestamp);
		},
		
		// Class properties
		properties:	{
			//TNPM1.3D: Extract Device Name from CounterID 371
		//    "Alcatel5529SDC_DeviceName": function(record) {
			    //TNPM1.3D: Commented out
				//return genDeviceName(record.descriptorName);
		//		if( record.CounterID == "371" && isValid(record.Value) ) {
		//			return record.Value;
		//		}
		//    },	
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
		    //TNPM1.3D: Extract Device IP from CounterID 370
		    "Alcatel5529SDC_DeviceIP": function(record) {			    	
		    	if( record.CounterID == "370" && isValid(record.Value) ) {
					return record.Value;
				}
		    },
            "Alcatel5529SDC_CardID": function(record) {
                 if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
                     return record.Value;
                 }
            },
			// Flag that the subelement conforms to the Alcatel5529SDC_Ethernet class
			"Alcatel5529SDC_EthernetLAG": function(record) {
				if (record.CounterID == "58" && record.Value == "ETHERNET")
					return "true";
			}
		},
		
		// Class formulas
		formulas: {
			"none": function(record) {
					return;
			}
		}
	}, // END Lag Related class
	"Alcatel5529SDC_IGMPCard": 
	{
		//The filter function returns true if a record conforms to the Alcatel5529SDC_LAGEthernet class
		filter: function(record) {
			// Since no records can be used from this particular function it will always return false,
			// however if the record is related to LAG then we'll add the isLagRelated = true;
			if( record.CounterID > 11150 && record.CounterID < 11167) {
				return true;                        
			}
			return false;
		},
		// The timestamp function returns the timestamp to use when creating metrics
        timestamp: function(record) {
			return (record.Timestamp);
        },
        // Class properties
        properties: {
        	//TNPM1.3D: Extract Device Name from CounterID 371
        // 	"Alcatel5529SDC_DeviceName": function(record) {
        		//TNPM1.3D: Commented out
        		//return genDeviceName(record.descriptorName);
        //		if( record.CounterID == "371" && isValid(record.Value) ) {
        //			return record.Value;
        //		}
        // 	},	
			//  PMR: 27994,012,649, revert back the above changes to the old way
			//  of populating the Device Name from the filename
			    "Alcatel5529SDC_DeviceName": function(mynewelement){
			        if(isValid(mynewelement)) {
			             return mynewelement.element.name;
					}
			    },
        	//TNPM1.3D: Extract Device IP from CounterID 370
        	"Alcatel5529SDC_DeviceIP": function(record) {			    	
        		if( record.CounterID == "370" && isValid(record.Value) ) {
        			return record.Value;
        		}
        	},
	        // Flag that the subelement was produce by the Alcatel5529SDC_UBAAdaptor provider
	        "Alcatel5529SDC_CardID": function(record) {
        		if ( record.CounterID.indexOf("22002") != -1 || record.CounterID.indexOf("5572") != -1 ) {
        			return record.Value;
        		}
	        },
            "Alcatel5529SDC_UBAAdaptor":  function(record) {
	        	return "true";
            },// Flag that this is an IGMP module
            "Alcatel5529SDC_IGMPModule":  function(record) {
            	return "true";
            },
            // Split the object into the shelf/slot/card for use as properties
            "Alcatel5529SDC_Port_Slot": function(record) {
            	if (record.CounterID == "11151") return record.ObjectID.split("-")[1];
            },
            "Alcatel5529SDC_Port_Shelf": function(record) {
            	if (record.CounterID == "11151") return record.ObjectID.split("-")[0];   
            },
            "Alcatel5529SDC_Port_Card": function(record) {
            	if (record.CounterID == "11151")
            		return record.ObjectID.substring(0, record.ObjectID.lastIndexOf("-"));
            },
            "Alcatel5529SDC_CARD": function(record) {
            	return "true";
            }
        },
        // Class formulas
        formulas: {
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~IGMP~Total IGMP messages received": function(record) {
        		if (record.CounterID == "11151"){
        			var formulaPath = arguments.callee.formulaPath;
        			return delta(record,formulaPath,true);
                }
            },
            "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~IGMP~Total join requests received": function(record) {
                if (record.CounterID == "11154") {
                	var formulaPath = arguments.callee.formulaPath;
                	return delta(record,formulaPath,true);
                }
            },
            "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~IGMP~Total successful join requests received": function(record) {
                if (record.CounterID == "11155") {
                	var formulaPath = arguments.callee.formulaPath;
                	return delta(record,formulaPath,true);
                }
            },
            "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~IGMP~Total failed join requests received": function(record) {
                if (record.CounterID == "11156") {
                	var formulaPath = arguments.callee.formulaPath;
                	return delta(record,formulaPath,true);
                }
            },
            "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~IGMP~Connects peak time per second": function(record) {
                if (record.CounterID == "11160")
                        return record.Value;
            },
            "AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Card~IGMP~Total multicast group counter": function(record) {
                if (record.CounterID == "11166")
                        return record.Value;
            }
        }
	},
	"Alcatel5529SDC_PON": 
	{
		//The filter function returns true if a record conforms to the Alcatel5529SDC_PON class
		filter: function(record) {
			if( record.CounterID == 11070 ) {
				return true;                        
			}
			return false;
		},
		// The timestamp function returns the timestamp to use when creating metrics
        timestamp: function(record) {
			return (record.Timestamp);
        },
        // Class properties
        properties: {
        	"Alcatel5529SDC_Port_Rack": function(record) {
	    		if( record.CounterID == "11070" ) {
	    			var val = record.ObjectID.split("-")[0];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_Port_Rack[PON] : " + val);
	    			return val;
	    		}
    		},
    		"Alcatel5529SDC_Port_Shelf": function(record) {
    			if( record.CounterID == "11070" ) {
	    			var val = record.ObjectID.split("-")[1];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_Port_Shelf[PON] : " + val);
	    			return val;
	    		}
    		},
    		"Alcatel5529SDC_Port_Slot": function(record) {
    			if( record.CounterID == "11070" ) {
	    			var val = record.ObjectID.split("-")[2];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_Port_Slot[PON] : " + val);
	    			return val;
	    		}
    		},
        	"Alcatel5529SDC_PON_ID": function(record) {
    			if( record.CounterID == "11070" ) {
	    			var val = record.ObjectID.split("-")[3];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_PON_ID[PON] : " + val);
	    			return val;
	    		}
        	}
    		,
        	"Alcatel5529SDC_PON": function(record) {
    			return true;
        	}
        }, 
        // Class formulas
        formulas: {
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~PON~Errored GEM Fragments": function(record) {
	    		if (record.CounterID == "11070"){
	    			logMessage(5, logComponent, "SubElementClasses","Errored GEM Fragments[PON] : " + record.Value);
	    			return record.Value;
	            }
        	}
        }
	},
	"Alcatel5529SDC_ONT": 
	{
		//The filter function returns true if a record conforms to the Alcatel5529SDC_PON class
		filter: function(record) {
			if( (151510 <= record.CounterID && record.CounterID <= 151514) ||
				(record.CounterID == 11071) ||
				(11077 <= record.CounterID && record.CounterID <= 11081) ||
				(11089 <= record.CounterID && record.CounterID <= 11095)
			){
				return true;                        
			}
			return false;
		},
		// The timestamp function returns the timestamp to use when creating metrics
        timestamp: function(record) {
			return (record.Timestamp);
        },
        // Class properties
        properties: {
        	"Alcatel5529SDC_Port_Rack": function(record) {
	    		if( record.CounterID == "151513" ) {
	    			var val = record.ObjectID.split("-")[0];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_Port_Rack[ONT] : " + val);
	    			return val;
	    		}
    		},
    		"Alcatel5529SDC_Port_Shelf": function(record) {
    			if( record.CounterID == "151513" ) {
	    			var val = record.ObjectID.split("-")[1];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_Port_Shelf[ONT] : " + val);
	    			return val;
	    		}
    		},
    		"Alcatel5529SDC_Port_Slot": function(record) {
    			if( record.CounterID == "151513" ) {
	    			var val = record.ObjectID.split("-")[2];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_Port_Slot[ONT] : " + val);
	    			return val;
	    		}
    		},
        	"Alcatel5529SDC_PON_ID": function(record) {
    			if( record.CounterID == "151513" ) {
	    			var val = record.ObjectID.split("-")[3];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_PON[ONT] : " + val);
	    			return val;
	    		}
        	},
        	"Alcatel5529SDC_ONT_ID": function(record) {
    			if( record.CounterID == "151513" ) {
	    			var val = record.ObjectID.split("-")[4];
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_ONT_ID[ONT] : " + val);
	    			return val;
	    		}
        	},
        	"Alcatel5529SDC_ONT": function(record) {
    			return true;
        	},
        	"Alcatel5529SDC_ONT_SLID": function(record) {
    			if( record.CounterID == "151511" ) {
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_ONT_SLID[ONT] : " + record.Value);
	    			return record.Value;
	    		}
        	},
        	"Alcatel5529SDC_ONT_Serial_No": function(record) {
    			if( record.CounterID == "151510" ) {
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_ONT_Serial_No[ONT] : " + record.Value);
	    			return record.Value;
	    		}
        	},
        	"Alcatel5529SDC_ONT_Version_No": function(record) {
    			if( record.CounterID == "151512" ) {
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_ONT_Version_No[ONT] : " + record.Value);
	    			return record.Value;
	    		}
        	},
        	"Alcatel5529SDC_ONT_Admin_State": function(record) {
    			if( record.CounterID == "151514" ) {
	    			logMessage(5, logComponent, "SubElementClasses","Alcatel5529SDC_ONT_Admin_State[ONT] : " + record.Value);
	    			return record.Value;
	    		}
        	}
        }, 
        // Class formulas
        formulas: {
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Blocks transmitted DS": function(record) {
	    		if (record.CounterID == "11077"){
	    			logMessage(5, logComponent, "SubElementClasses","Blocks transmitted DS : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Blocks received US": function(record) {
	    		if (record.CounterID == "11079"){
	    			logMessage(5, logComponent, "SubElementClasses","Blocks received US : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Frames received US": function(record) {
	    		if (record.CounterID == "11080"){
	    			logMessage(5, logComponent, "SubElementClasses","Frames received US : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Frames transmitted US": function(record) {
	    		if (record.CounterID == "11078"){
	    			logMessage(5, logComponent, "SubElementClasses","Frames transmitted US : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Lost frames US": function(record) {
	    		if (record.CounterID == "11081"){
	    			logMessage(5, logComponent, "SubElementClasses","Lost frames US : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Received GEM fragments": function(record) {
	    		if (record.CounterID == "11091"){
	    			logMessage(5, logComponent, "SubElementClasses","Received GEM fragments : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Received GEM blocks": function(record) {
	    		if (record.CounterID == "11092"){
	    			logMessage(5, logComponent, "SubElementClasses","Received GEM blocks : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Transmitted GEM blocks": function(record) {
	    		if (record.CounterID == "11093"){
	    			logMessage(5, logComponent, "SubElementClasses","Transmitted GEM blocks : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Transmitted GEM blocks": function(record) {
	    		if (record.CounterID == "11093"){
	    			logMessage(5, logComponent, "SubElementClasses","Transmitted GEM blocks : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Transmitted GEM fragments": function(record) {
	    		if (record.CounterID == "11094"){
	    			logMessage(5, logComponent, "SubElementClasses","Transmitted GEM fragments : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Errored GEM fragments": function(record) {
	    		if (record.CounterID == "11071"){
	    			logMessage(5, logComponent, "SubElementClasses","Errored GEM fragments : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Lost GEM fragments DS": function(record) {
	    		if (record.CounterID == "11089"){
	    			logMessage(5, logComponent, "SubElementClasses","Lost GEM fragments DS : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Lost GEM fragments US": function(record) {
	    		if (record.CounterID == "11090"){
	    			logMessage(5, logComponent, "SubElementClasses","Lost GEM fragments US : " + record.Value);
	    			return record.Value;
	            }
        	},
        	"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~ONT~Bad received GEM headers": function(record) {
	    		if (record.CounterID == "11095"){
	    			logMessage(5, logComponent, "SubElementClasses","Bad received GEM headers : " + record.Value);
	    			return record.Value;
	            }
        	}
        }
	}
};


/*****
 * addes the FormulaPath to each function so that it can be referenced by arguments.callee.formulaPath
 * @param {Object} allSubElementClasses
 */
function addFormulaPathToAllFormulaFunctions(allSubElementClasses)
{
for ( var className in allSubElementClasses ) 
	{
		if(allSubElementClasses.hasOwnProperty(className)) {
			var tmpClass = allSubElementClasses[className];
			for ( var formulaName in tmpClass.formulas )
			{
				if (tmpClass.formulas.hasOwnProperty(formulaName))
				{
					tmpClass.formulas[formulaName].formulaPath=formulaName;
				}
				
			}
		}
	}
}

/**
 * adds the object of isLagRelated=true if the record is determined to be desired by the LagAggregator
 * @param {PV.Record} record
 */
function addIsLagRelatedToRecord(record){
	// 90001-90013 == MIB-2 stats
	// 121212-121235 LAG specific stats
	//logMessage(5, logComponent, "addIsLagRe","Looking at record" );
        if ( PV.configuration.CARDLEVEL != "TRUE" ) {
	   if ( (90001 <= record.CounterID && record.CounterID < 90014) ||
		(121212 <= record.CounterID && record.CounterID < 121241 ) ||
		(80001 <= record.CounterID && record.CounterID < 80005 ) || 
		(370 <= record.CounterID && record.CounterID < 372) ){
		record.isLagRelated=true;
	   }
        }
}

addFormulaPathToAllFormulaFunctions(subElementClasses);
