// ******************************************************************************
// * Licensed Materials - Property of IBM
// * 5724-W86, 5724-X63
// *  Copyright IBM Corporation 2008, 2012. All Rights Reserved.
// * US Government Users Restricted Rights- Use, duplication or disclosure
// * restricted by GSA ADP Schedule Contract with IBM Corp.
// ******************************************************************************
// *
// ******************************************************************************


/**
 * specialty aggregator for calculating LAG metrics. There is an impiled assumption
 *  of one instance of this aggergator per element
 * @param {PV.Operator} parent to this aggregation
 */
function createLinkGroupAggregator(operator){
	var aggregator = operator.Aggregator();
	aggregator.setPeriodDuration(900);
	aggregator.parentOperator=operator;
	aggregator.persistent=true;
	aggregator.name="LAGAggregator";

     //Initialize aggregator with inventory on resync
     //parentOperator.onResync(function() { aggregator.initializeWithInventory(); });

     //Debugging
     // if (LOG_DEBUG) result.traceAll=true;

     /*
      * Use subelement name as the key
      */
     aggregator.keyFunction=function(record)
     {
	 	// The default key is unknown. since all other keys will be
		// MAC addresses "FE:DC:BA:98:76:54" this should never overlap
	 	 var returnKeyString = "unknown";
		 if(!isValid(this.macFromIPLookupTable)) {
		 	this.macFromIPLookupTable = new Object();
		 }
		 if(isValid(this.macFromIPLookupTable[record.subelement.name])){
		 	returnKeyString = this.macFromIPLookupTable[record.subelement.name];
		 }
         return returnKeyString;
     },

	 /*
      * Initialize the state for the first record seen
      */
     aggregator.initializeFunction=function(record) 
     {
         var result=new Object();
		 result.usesLagIdentfier= false;
		 result.periodDefinition=this.periodDuration;
         //this.initializeState(result);
         return result;
     },
     
     /*
      * Calculate state for record
      */
    aggregator.calculateFunction=function(key, state, record) 
     {
	 	if(record.isLagRelated)
		{
			if(record.CounterID == 121216 && ! state.usesLagIdentfier) {
				logMessage(5, logComponent, "LAGcalculatef..","Found a MAC address :"+ record.Value);
				var newkey=record.element.name +":" +record.Value;
				
				if(key == "unknown") {
					this.moveStateFromUnknownToNewKey(state,newkey, record);
					this.macFromIPLookupTable[record.subelement.name]=newkey;
				} // Endif (key==unkown)
				if( (! state.usesLagIdentfier) && key != "unknown" && (key != newkey)) {
					// this lag has moved, let's try to keep up
					//TODO you use record.element.name +":" +record.Value alot, maybe only make it once?
					this.moveStateToNewKey(state.key,state, newkey, record);
					this.macFromIPLookupTable[record.subelement.name]=newkey;
				} //EndIF (key != "unknown" && key != record.Value)
				// if(key == record.Value) 	// Nothing we're where we want to be
			} // EndIF f(record.CounterID == 121216 ) 
			else
			if(record.CounterID == 121240) {
				logMessage(5, logComponent, "LAGcalculatef..","Found a LAG Identifier :"+ record.Value);
				state.usesLagIdentfier = true;
				var newkey=record.element.name +":LAG_ID_" +record.Value;
				if(key == "unknown") {
					this.moveStateFromUnknownToNewKey(state,newkey, record);
					this.macFromIPLookupTable[record.subelement.name]=newkey;
				} else if( key != "unknown" && (key != newkey)) {
					// this lag has moved, let's try to keep up
					this.moveStateToNewKey(state.key,state, newkey, record);
					this.macFromIPLookupTable[record.subelement.name]=newkey;
				}
				if( newkey != state.key){
					state.key=newkey;
				}
				if(! isValid(state.elementName)) {
					state.elementName = record.element.name;
				}
			}
			else 
			{
				//we have an object ID that could be a metric.
				if(key == "unknown") {
					//add this metrics to state[seName].currentPeriod[CounterID]
					if(! isValid(state[record.subelement.name]))
						state[record.subelement.name] = new Object();
					if(! isValid(state[record.subelement.name].currentPeriod))
						state[record.subelement.name].currentPeriod=new Object();
					state[record.subelement.name].currentPeriod[record.CounterID]=record.Value;
					
				} else {
					// add this metric to state.currentPeriod[CounterID][ObjectID]
					if(! isValid(state.currentPeriod))
						state.currentPeriod = new Object();
					if(! isValid(state.currentPeriod[record.CounterID]))
						state.currentPeriod[record.CounterID]=new Object();
					state.currentPeriod[record.CounterID][record.ObjectID]=record.Value;
				}
			}
		}
          
	},

     /*
      * The period is closed and we need to flush our metrics and reset our state for the next period
      */
     aggregator.closePeriodFunction=function(state) 
     {
		if(state.key != "unknown") {
				if(isValid(state.currentPeriod) ) 
					{
						calculateLag(this.parentOperator,state.elementName,state.key,state,(this.startOfNextPeriod - 1));
								
					} //  if(isValid(state.currentPeriod) ) 
				state.previousPeriod = state.currentPeriod;
				state.currentPeriod = undefined;
		} //endIf (state.key != "unknown")
		else 
		{
			for(seName in state) 
			{
				if(state.hasOwnProperty(seName))
				 {
					//I didn't use any of this, I don't need it.
					state[seName].currentPeriod = undefined;
				}
			}
				
				
		}
     },
	 /**
	  * 
	  * @param {Object} oldkey
	  * @param {Object} state
	  * @param {Object} newkey
	  * @param {Object} record
	  */
	 aggregator.moveStateToNewKey=function(oldkey, state, newkey, record) {
	 	var destState = this.getStateAt(newkey);
		destState.key = newkey;
		destState.elementName = record.element.name;
		if(! isValid(destState.currentPeriod)) {
			destState.currentPeriod = new Object();
		}
		// add this port to that newState
		// move the port metrics from the old state to the new state
		if(isValid(state.currentPeriod) ) 
		{
			for( var CounterID in state.currentPeriod )
			{
				if(state.currentPeriod.hasOwnProperty(CounterID))
				{
					for( var ObjectID in state.currentPeriod[CounterID]) {
						if(state.currentPeriod[CounterID].hasOwnProperty(ObjectID))
						{
							if(ObjectID == record.ObjectID) {
								if(!isValid(destState.currentPeriod)) {
									destState.currentPeriod = new Object();
								}
								if(!isValid(destState.currentPeriod[CounterID])) {
									destState.currentPeriod[CounterID] = new Object();
								}
								destState.currentPeriod[CounterID][ObjectID] = state.currentPeriod[CounterID][ObjectID];
								state.currentPeriod[CounterID][ObjectID] = undefined;
							}
						}
					}
				}
			}
		} //  if(isValid(state.currentPeriod) ) 
	 },

	aggregator.moveStateFromUnknownToNewKey=function(state,newkey,record){
		var destState = this.getStateAt(newkey);
		destState.key = newkey;
		destState.elementName = record.element.name;
		if(! isValid(destState.currentPeriod)) {
			destState.currentPeriod = new Object();
		}
		// move the state from state[seName][CounterID]
		if(isValid(state[record.subelement.name])) {
			if(isValid(state[record.subelement.name].currentPeriod))
				{
				for( var CounterID in state[record.subelement.name].currentPeriod )
					{
						if(state[record.subelement.name].currentPeriod.hasOwnProperty(CounterID))
						{
							// move each object
							if(!isValid(destState.currentPeriod[CounterID]))
								destState.currentPeriod[CounterID] = new Object();
							destState.currentPeriod[CounterID][record.ObjectID] = state[record.subelement.name].currentPeriod[CounterID];
						}
					}
				}
		}
		// remove state[seName]
		state[record.subelement.name] = undefined;								
	};
	
     /*
      * Initialize the state (not needed)
      */
     //aggregator.initializeState=function(state)
    // {
		//TODO do we need anything here?
		// might be a good place to put the key.
     //}


//     result.keyFunction=logBeforeAfter("keyFunction",result.keyFunction);
//     result.initializeFunction=logBeforeAfter("initializeFunction",result.initializeFunction);
//     result.calculateFunction=logBeforeAfter("calculateFunction",result.calculateFunction);
//     result.closePeriodFunction=logBeforeAfter("closePeriodFunction",result.closePeriodFunction);
//     result.initializeState=logBeforeAfter("initializeState",result.initializeState);
//     result.initializeWithInventory=logBeforeAfter("initializeWithInventory",result.initializeWithInventory);
	
	//aggregator.traceTuples=true;
	return aggregator;
	
}

/**
 * Calculate all LAGs and submit them for a given MAC
 * @param {PV.Operator} operator
 * @param {String} elementName
 * @param {String} subelementName
 * @param {Object} state
 * @param {Date} timestamp
 */
function calculateLag(operator,elementName,subelementName,state,timestamp)
{
	logMessage(7, logComponent, "calculateLag","LAG:" + subelementName);
	if(state.usesLagIdentfier) {
		logMessage(5, logComponent, "calculateLag","LAG_ID:" + state.key);
	}
	var masterID = findLagMaster(state);
	if (masterID.length > 0 ) {
			state.masterPort = masterID;
			logMessage(8, logComponent, "calculateLag","Master of LAG: " + state.masterPort);
	}
	addEthernetPropertyOfLagName(operator,elementName,state,subelementName);
	if(isValid(state.masterPort)) 
	{
	var element = operator.elementNamed(elementName);
	state.currentTimestamp = timestamp;
	var subelementObject=SubelementInRecord.clone().mixIn({
		instance:	subelementName,
		invariant:	subelementName,
		label:		"LAG Master " + state.masterPort,
		name:		subelementName
	});
	var subelement = operator.subelementNamedOrNew(subelementName);
	enhanceModelFromObject(subelement,subelementObject);
	subelement.element = element;
	
	// This is because we must have created the element for the ethernet port already
	//processProperties(LAGClass["Alcatel5529SDC_EthernetLAG"], subelement, state)
	processProperties(LAGClass["Alcatel5529SDC_EthernetLAG"], subelement, state);
	
	processFormulasWithOperator(operator,LAGClass["Alcatel5529SDC_EthernetLAG"], subelement, state);
	}
}

function genDeviceName(descr) {
       var name = descr.substring(0, descr.lastIndexOf("_"));
       return name;
}

var LAGClass = {
	"Alcatel5529SDC_EthernetLAG": {
		// The filter function returns true if a record conforms to the Alcatel5529SDC_LAGEthernet class
		filter: function(state) {
		// Since no records can be used from this particular function it will always return false,
		// however if the record is related to LAG then we'll add the isLagRelated = true;
			return isValid(state.key);
		},
		
		// The timestamp function returns the timestamp to use when creating metrics
		timestamp: function(state) {
                        return state.currentTimestamp;
		},
		
		// Class properties
		properties:	{
			"Alcatel5529SDC_UBAAdaptor": function(state) 
			{
				return "true";
			},
			"Alcatel5529SDC_Port": function(state) 
			{
				return "true";
			},
			// Flag that the subelement conforms to the Alcatel5529SDC_EthernetLAG class
			"Alcatel5529SDC_EthernetLAG": function(state)
			{
					return "true";
			},
			"Alcatel5529SDC_EthernetLAG_Speed": function(state)
			{
					return sumCurrentPeriod(state,90011);
			},
			//TNPM1.3D: Extract Device Name from CounterID 371
		//    "Alcatel5529SDC_DeviceName": function(state) {
		//		var counterID = 371;
		//		var val	= getCounterValueFromStateAndMasterPort(state,counterID);
		//		if(isValid(val)){
		//			logMessage(5, logComponent, "LAGClass","Alcatel5529SDC_DeviceName : " + val);
		//			return val;
		//		}
		//   },	
			// PMR: 27994,012,649, revert back the above changes to the old way
			// of populating the Device Name from the filename
			"Alcatel5529SDC_DeviceName": function(state) {
				return state.elementName;
			},
		    //TNPM1.3D: Extract Device IP from CounterID 370
		   "Alcatel5529SDC_DeviceIP": function(state) {			    	
		  	var counterID = 370;
				var val	= getCounterValueFromStateAndMasterPort(state,counterID);
				if(isValid(val)){
					logMessage(5, logComponent, "LAGClass","Alcatel5529SDC_DeviceIP : " + val);
					return val;
				}
		   },
			"Alcatel5529SDC_LAG_Name": function(state)
			{
				var counterID = 121229;
				var val	= getCounterValueFromStateAndMasterPort(state,counterID);
				if(isValid(val)){
					logMessage(5, logComponent, "LAGClass","Alcatel5529SDC_LAG_Name : " + val);
					return val;
				}
			},
			"Alcatel5529SDC_LAG_Rack": function(state)
			{
				var val	= state.masterPort;
				if(isValid(val)){
					var info 	= val.substring(val.lastIndexOf(" "));
					var rack 	= info.split("-")[0];
					if( isValid(rack) ) {
						logMessage(5, logComponent, "LAGClass","Alcatel5529SDC_LAG_Rack : " + rack);
						return rack;
					}
				}				
			},
			"Alcatel5529SDC_LAG_Shelf": function(state)
			{
				var val	= state.masterPort;
				if(isValid(val)){
					var info 	= val.substring(val.lastIndexOf(" "));
					var shelf 	= info.split("-")[1];
					if( isValid(shelf) ) {
						logMessage(5, logComponent, "LAGClass","Alcatel5529SDC_LAG_Shelf : " + shelf);
						return shelf;
					}
				}					
			},
			"Alcatel5529SDC_LAG_Slot": function(state)
			{
				var val	= state.masterPort;
				if(isValid(val)){
					var info 	= val.substring(val.lastIndexOf(" "));
					var slot 	= info.split("-")[2];
					if( isValid(slot) ) {
						logMessage(5, logComponent, "LAGClass","Alcatel5529SDC_LAG_Slot : " + slot);
						return slot;
					}
				}
			},
			"Alcatel5529SDC_LAG_Port": function(state)
			{
				var val	= state.masterPort;
				if(isValid(val)){
					var info 	= val.substring(val.lastIndexOf(" "));
					var port 	= info.split("-")[3];
					if( isValid(port) ) {
						logMessage(5, logComponent, "LAGClass","Alcatel5529SDC_LAG_Port : " + port);
						return port;
					}
				}
			}	
		},
		
		// Class formulas
		formulas: {
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming discarded packets": function(state) {
				//return simpleLagDelta(state,121222);
				return simpleLagDelta(state,90003);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming multicast packets": function(state) {
				return simpleLagDelta(state,80001);
			}, 
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming broadcast packets": function(state) {
				return simpleLagDelta(state,80003);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming packets": function(state) {
				//return simpleLagDelta(state,121232);
				return (simpleLagDelta(state,90007) * 1 + simpleLagDelta(state,90012)); 
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming octets": function(state) {
				//return simpleLagDelta(state,121220);
				return simpleLagDelta(state,90001);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming packet errors": function(state) {
				//return simpleLagDelta(state,121223);
				return simpleLagDelta(state,90005);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming unicast packets": function(state) {
				//return simpleLagDelta(state,121221);
				return simpleLagDelta(state,90007);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing discarded packets": function(state) {
				//return simpleLagDelta(state,121227);
				return simpleLagDelta(state,90004);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing packets": function(state) {
				//return simpleLagDelta(state,121235);
				return (simpleLagDelta(state,90008) * 1 + simpleLagDelta(state,90013)); 
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing multicast packets": function(state) {
					return simpleLagDelta(state,80002);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing broadcast packets": function(state) {
					return simpleLagDelta(state,80004);
			}, 
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing octets": function(state) {
					//return simpleLagDelta(state,121225);
					return simpleLagDelta(state,90002);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing packet errors": function(state) {
					//return simpleLagDelta(state,121228);
					return simpleLagDelta(state,90006);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing unicast packets": function(state) {
					//return simpleLagDelta(state,121226);
					return simpleLagDelta(state,90008);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Unknown protocol errors": function(state) {
					//return simpleLagDelta(state,121224);
					return simpleLagDelta(state,90010);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing non-unicast packets": function(state) {
					//return (simpleLagDelta(state,121234) * 1 + simpleLagDelta(state,121233)); 
					return simpleLagDelta(state,90013);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming non-unicast packets": function(state) {
					//return (simpleLagDelta(state,121230) * 1 + simpleLagDelta(state,121231)); 
					return simpleLagDelta(state,90012);
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Incoming utilization": function(state) {
					var speed = sumCurrentPeriod(state,90011) *1;
					var octets = simpleLagDelta(state,90001) *1;
					var time = state.periodDefinition *1;
					var returnValue =  800 * (1* octets) / ( 1 * speed * time ) ;
					logMessage(5, logComponent, "calculateLag","LAG utilization detected:100 * " + 
						octets + " / ( " + speed + " * " + time + " ) = " + returnValue );
					return returnValue;
			},
			"AP~Specific~Bulk~Alcatel-Lucent 5529 SDC~UBAAdaptor~Port~Ethernet LAG~Outgoing utilization": function(state) {
					//return (simpleLagDelta(state,121230) * 1 + simpleLagDelta(state,121231)); 
					var speed = sumCurrentPeriod(state,90011) *1;
					var octets = simpleLagDelta(state,90002) *1;
					var time = state.periodDefinition *1;
					var returnValue =  800 * (1* octets) / ( 1 * speed * time ) ;
					logMessage(5, logComponent, "calculateLag","LAG utilization detected:100 * " + 
						octets + " / ( " + speed + " * " + time + " ) = " + returnValue );
					return returnValue;
			}
		}
	} // END Lag Related class
};

/**
 * Calculates the lag for a specific counter by subtracting the previous period from the current period
 * @param {Object} state
 * @param {String} counterID
 */
function simpleLagDelta(state,counterID) {
	var returnValue;
	if(isValid(state.currentPeriod[counterID]))
		if(isValid(state.previousPeriod)) {
			if(! isValid(returnValue)) returnValue= 0;
			if(isValid(state.previousPeriod[counterID])) {
				for(var port in state.currentPeriod[counterID]){
					if(state.currentPeriod[counterID].hasOwnProperty(port))
					if(isValid(state.previousPeriod[counterID][port]))
					{
						if((1 * state.currentPeriod[counterID][port]) >= (1 * state.previousPeriod[counterID][port]))
							returnValue += state.currentPeriod[counterID][port] - state.previousPeriod[counterID][port];
						else {
							//TODO we need to do something with this counter that's negative, 
							//Assuming a daily rollover
							returnValue= 0;
							logMessage(4, logComponent, "calculateLag","LAG Rollover detected: " + state.key + ", counterID: " + counterID + "setting to zero");
						}
					}
				}
			}
		}
	return returnValue;
}
/**
 * Returns the value for a sum of values (across ports) for a single counterID
 * @param {Object} state
 * @param {Object} counterID
 */
function sumCurrentPeriod(state,counterID) {
	var returnValue;
	if(isValid(state.currentPeriod[counterID]))
	{
		if(! isValid(returnValue))
			returnValue= 0;
		for(var port in state.currentPeriod[counterID])
		{
			if(state.currentPeriod[counterID].hasOwnProperty(port))
			{
				var previousValue; 
				if(isValid(state.currentPeriod[counterID][port])) {
					previousValue = returnValue * 1;
					var currentValue = 1 * previousValue + (state.currentPeriod[counterID][port] * 1);
					returnValue = currentValue;
				}
			}
		}	
	}
	return returnValue;
}
/**
 * Return the first value(across ports) for a single counterID
 * @param {Object} state
 * @param {Object} counterID
 */
function getCounterValueFromState( state, counterID) {
	if(isValid(state.currentPeriod[counterID])){
		//Might have more objID but value should be same so return the first
		for( var objID in state.currentPeriod[counterID] ) {
			if( state.currentPeriod[counterID].hasOwnProperty(objID) ) {
				var val = state.currentPeriod[counterID][objID];
				if(isValid(val)){
					logMessage(5, logComponent, "getCounterValueFromState","Counter[" + counterID + "] : " + val);
					return val;
				}
			}
		}
	} else {
		logMessage(5, logComponent, "getCounterValueFromState","Counter[" + counterID + "] not exist.");
	}
}
/**
 * Return the value of single port (based on the state.masterPort) for a single counterID
 * @param {Object} state
 * @param {Object} counterID
 */
function getCounterValueFromStateAndMasterPort( state, counterID) {
	if(isValid(state.currentPeriod[counterID])){		
		if( state.currentPeriod[counterID].hasOwnProperty(state.masterPort) ) {
			var val = state.currentPeriod[counterID][state.masterPort];
			if(isValid(val)){
				logMessage(5, logComponent, "getCounterValueFromStateAndMasterPort","Port: " + state.masterPort + ", Counter: " + counterID + ", value: " + val);
				return val;
			}
		} else {
			logMessage(5, logComponent, "getCounterValueFromStateAndMasterPort","Port: " + state.masterPort + ", Counter: " + counterID + ", value: not exist.");
		}
	} else {
		logMessage(5, logComponent, "getCounterValueFromStateAndMasterPort","Counter[" + counterID + "] not exist.");
	}
}
/**
 * walks the entire subElementClass passing the record (state) into each one, if a result
 * is returned it will create a metric for it.
 * @param {PV.Operator} operator
 * @param {Object} subElementClass
 * @param {PV.Subelement} subElement
 * @param {Object} record
 */
function processFormulasWithOperator(operator,subElementClass, subElement, record) {
//	var timestamp = subElementClass.timestamp(record) * 1;
//	above is commented out because we seem to be using date objects for metric 
//	timestamps now instead of epoch time
	var timestamp = subElementClass.timestamp(record);
	for ( var formulaName in subElementClass.formulas ) 
	{
		if(subElementClass.formulas.hasOwnProperty(formulaName))
		{
			var value = subElementClass.formulas[formulaName](record) * 1;
			addMetricWithOperator(operator,subElement, formulaName, timestamp, value);
		}
	}
}

/**
 * finds and returns the master of the lag
 * @param {Object} state
 * @return {String} Lag port Identifier
 */
function findLagMaster(state){
	//we currently can only be assured of a master if counter 121212 has the text:
	//The number of ports attached to this Aggregator is : <int>
	// where int is non-zero
	var counterID=121212;
	var returnValue = "";
	if(isValid(state.currentPeriod))
		if(isValid(state.currentPeriod[counterID]))
			for(var port in state.currentPeriod[counterID])
			{
				if(state.currentPeriod[counterID].hasOwnProperty(port))
				{
					if(isValid(state.currentPeriod[counterID][port]))
					{
						var value = state.currentPeriod[counterID][port];
						var integer= value.slice(value.lastIndexOf(":") + 2, value.length);
						if((integer*1 ) > 0)
							{
								logMessage(5, logComponent, "calculateLag","findLagMaster found a Master:" + port );
								returnValue = port;
							}
					}
				}
			}	
	return returnValue;
}

/**
 * takes the state "regardless" of lag and sets the lag master property
 * @param {PV.Operator} operator
 * @param {String} elementName
 * @param {String} subelementName
 * @param {Object} state
 * @param {Date} timestamp
 */
function addEthernetPropertyOfLagName(operator,elementName,state,subelementName)
{
	var counterID=121212;
	var lagName = "n/a";
	if(isValid(state.masterPort))
		lagName = state.masterPort;
	//logMessage(5, logComponent, "addEther...","called" );
	if(isValid(state.currentPeriod)){
		logMessage(5, logComponent, "addEther...","state has a Master: " +lagName );
		//This is a lag, set the master on the children
		if(isValid(state.currentPeriod[counterID]))
			for(var port in state.currentPeriod[counterID])
			{
				if(state.currentPeriod[counterID].hasOwnProperty(port))
				{
						// This seems like alot of iteration, but its the only way to be sure
						var portSEName = elementName + ":" +port;
						logMessage(5, logComponent, "addEther...","found port in a Master for subelement: " + port +" in " + lagName  + " for " +portSEName);
						var subelement = operator.subelementNamed(portSEName);
						logMessage(5, logComponent, "addEther...","fetched subelement: " + portSEName);
						logMessage(5, logComponent, "addEther...","subelement fetched is: " + subelement);
						// I believe this does not work on 4.4.1.4
						//if( isValid(subelement)) {
						if( subelement != undefined ) {
							logMessage(5, logComponent, "addEther...","adding port in a Master: " + port +" in " + lagName );
							addProperty(subelement, "Alcatel5529SDC_Ethernet_LagMaster", lagName);
							logMessage(5, logComponent, "addEther...","completed");
						}
				}
			}
	} 
	logMessage(5, logComponent, "addEther...","successful completion" );
}
